import java.util.LinkedList;

import java.util.Arrays;

public class SternBrocot {

	private int p, q; // numérateur, dénominateur
	private SternBrocot left, right;

	SternBrocot(SternBrocot left, int p, int q, SternBrocot right) {
		this.p = p;
		this.q = q;
		this.left = left;
		this.right = right;
	}


	// seulement utilisé dans assert
	static int gcd(int n, int m) {
		return m == 0 ? n : gcd(m, n % m);
	}
	

	// construit l'arbre de Stern-Brocot des rationnels p/q avec
	// pl/ql < p/q < pr/qr et q <= limit
	static SternBrocot build(int pl, int ql, int pr, int qr, int limit) {
		assert pl * qr < pr * ql;
		assert gcd(pl, ql) == 1;
		assert gcd(pr, qr) == 1;
		// A completer:
		// BEGIN_CUT
		int p = pl + pr, q = ql + qr;
		if (q > limit)
			return null;
		return new SternBrocot(build(pl, ql, p, q, limit), p, q, build(p, q, pr, qr, limit));
		// END_CUT
	}

	// toutes les fractions réduites entre 0 et 1 de dénominateur <= limit
	static SternBrocot build(int limit) {
		return build(0, 1, 1, 1, limit);
	}



	static int size(SternBrocot t) {
		// BEGIN_CUT
		if (t == null)
			return 0;
		return 1 + size(t.left) + size(t.right);
		// END_CUT
	}


	static boolean isBST(SternBrocot t, int pl, int ql, int pr, int qr) {
		// BEGIN_CUT
		if(t == null)
			return true;
		if(t.p * ql < pl * t.q || t.p * qr > pr * t.q)
			return false;
		return isBST(t.left, pl, ql, t.p, t.q) && isBST(t.right, t.p, t.q, pr, qr);
		// END_CUT
	}

	// BEGIN_CUT
	static void treeToList(SternBrocot t, LinkedList<String> l) {
		if(t == null)
			return;
		treeToList(t.left, l);
		l.add(t.p + "/" + t.q);
		treeToList(t.right, l);
	}
	// END_CUT


	static LinkedList<String> treeToList(SternBrocot t){
		// BEGIN_CUT
		LinkedList<String> result = new LinkedList<String>();
		treeToList(t, result);
		return result;
		// END_CUT
	}


	// le chemin (L pour left, R pour right) menant à x dans t
	// se termine par "!" si on trouve x dans t, sinon "?"
	static String path(SternBrocot t, int p, int q) {
		// BEGIN_CUT
		if (t == null)
			return "?";
		if (p * t.q == q * t.p)
			return "!";
		return p * t.q < q * t.p ? "L" + path(t.left, p, q) : "R" + path(t.right, p, q);
		// END_CUT
	}	

	
	static int[] fromPath(String path) {
		// BEGIN_CUT
		int pl = 0, ql = 1, pr = 1, qr = 1;
		for(int i = 0; i < path.length(); i++) {
			int p = pl + pr, q = ql + qr;
			if(path.charAt(i) == 'L') {
				pr = p;
                                qr = q;
			}
			if(path.charAt(i) == 'R') {
				pl = p;
				ql = q;
			}
		}
		return new int[] {pl + pr, ql + qr};
		// END_CUT
	}

	
	
	// la meilleure approximation rationnelle de x par défaut présente dans t
	// renvoyée comme un tableau de deux entiers
	static int[] lowApprox(SternBrocot t, double x) {
		// BEGIN_CUT
		if (t == null)
			return null;
		if (x * t.q < t.p)
			return lowApprox(t.left, x);
		int[] candidate = lowApprox(t.right, x);
		return candidate == null ? new int[] {t.p, t.q} : candidate;
		// END_CUT
	}

	
	// BEGIN_CUT
	
	static int[] upApprox(SternBrocot t, double x) {
		if (t == null)
			return null;
		if(x * t.q > t.p)
			return upApprox(t.right, x);
		int[] candidate = upApprox(t.left, x);
		return candidate == null ? new int[] {t.p, t.q} : candidate;		
	}
	
	
	
	public static void main(String[] args) {
		System.out.println("--- Test approx");
		SternBrocot t = SternBrocot.build(100);
		double d, eps1, eps2;
		int[] a, b;
		int counter = 0;
		int nTests = 1000;
		for(int k = 0; k < nTests; k++) {
			d = .01 + .98 * Math.random();
			a = upApprox(t, d);
			b = lowApprox(t, d);
			eps1 = d - a[0] * 1./a[1];
			eps2 = -d + b[0] * 1./b[1];
			if(eps1 >= 1./(a[1]*a[1]) || eps2 >= 1./(b[1] * b[1]))
				counter++;
		}
		System.out.println("Pour " + nTests + " tests, le nombre de mauvaises"
				+ " approximations est de : " + counter +".");

	}
	//END_CUT


	// ---- Seulement pour les tests ------
	// --------- Ne pas modifier ----------

	static boolean treeEquality(SternBrocot t1, SternBrocot t2) {
		if(t1 == null && t2 == null)
			return true;
		if(t1 == null || t2 == null)
			return false;
		return (t1.p == t2.p && t1.q == t2.q && treeEquality(t1.left, t2.left)
				&& treeEquality(t1.right, t2.right));
	}



}
